<?php
/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 * 
 * Local Version
 */
class MM_License
{	
    private $id;
	private $memberId;
	private $name;
	private $url;
	private $apiKey;
	private $authKey;
	private $apiSecret;
	private $majorVersion;
	private $minorVersion;
	private $permissionsProfileId;
	private $permissions;
	private $status;
	private $isArchivedFlag;
	private $isStaging;
	private $lastUpdated;
	private $dateAdded;
	private $isUnlicensed = false;
	
	protected static $license = null;
	
	public function __construct()
	{
	}
	
	public function initialize()
	{
		MM_MemberMouseService::getLicense($this);
	}
	
	/**
	 * Fetch and return the current license
	 * 
	 * @return MM_License An MM_License representing the current active license
	 */
	public static function getCurrentLicense()
	{
	    if (self::$license == null)
	    {
	        self::$license = new MM_License();
	        self::$license->initialize();
	    }
	    return self::$license;
	}
	
	public function setData($data)
	{
		$this->id = $data->id;
		$this->memberId = $data->memberId;
		$this->name = $data->name;
		$this->url = $data->url;
		$this->apiKey = $data->apiKey;
		$this->apiSecret = $data->apiSecret;
		$this->majorVersion = $data->majorVersion;
		$this->minorVersion = $data->minorVersion;
		$this->permissionsProfileId = $data->permissionsProfileId;
		$this->permissions = $data->permissions;
		$this->status = $data->status;
		$this->isArchivedFlag = $data->isArchivedFlag;
		$this->isStaging = $data->isStaging;
		$this->dateAdded = $data->dateAdded;
		$this->lastUpdated = $data->lastUpdated;	
	}
	
	public function commitData() {}
	
	
	/** GETTERS / SETTERS **/
	
	public function setId($id)
	{
	    $this->id = $id;
	}
	
	public function getId()
	{
	    return $this->id;
	}
	
	public function getAuthKey()
	{
	    return $this->authKey;
	}
	
	public function setAuthKey($authKey)
	{
	    $this->authKey = $authKey;
	}
	
	public function setMemberId($str)
	{
		$this->memberId = $str;
	}
	
	public function getMemberId()
	{
		return $this->memberId;
	}
	
	public function setName($str) 
	{
		$this->name = $str;
	}
	
	public function getName() 
	{
		return $this->name;
	}
	
	public function setUrl($str) 
	{
		$this->url = $str;
	}
	
	public function getUrl() 
	{
		return $this->url;
	}
	
	public function setApiKey($str)
	{
		$this->apiKey = $str;
	}
	
	public function getApiKey()
	{
		return $this->apiKey;
	}
	
	public function setApiSecret($str)
	{
		$this->apiSecret = $str;
	}
	
	public function getApiSecret()
	{
		return $this->apiSecret;
	}
	
	public function setMajorVersion($str) 
	{
		$this->majorVersion = $str;
	}
	
	public function getMajorVersion() 
	{
		return $this->majorVersion;
	}
	
	public function setMinorVersion($str) 
	{
		$this->minorVersion = $str;
	}
	
	public function getMinorVersion() 
	{
		return $this->minorVersion;
	}
	
	public function setPermissionsProfileId($str) 
	{
		$this->permissionsProfileId = $str;
	}
	
	public function getpermissionsProfileId() 
	{
		return $this->permissionsProfileId;
	}
	
	public function getLicenseType($permissionsProfileId="")
	{
	    if (empty($permissionsProfileId) || !is_numeric($permissionsProfileId))
	    {
	        $permissionsProfileId = $this->getpermissionsProfileId();
	    }
	    
	    $licenseTypes = [ 
	        -1 => "Custom",
	    	0  => "Unlicensed",
	        1  => "Free (Legacy)",
	        2  => "Starter V1 (Legacy)",
	        3  => "Growth V1 (Legacy)",
	        4  => "Professional V1 (Legacy)",
	        5  => "Starter (Legacy)",
	        6  => "Builder (Legacy)",
	        7  => "Growth (Legacy)",
	        10 => "Advanced (Legacy)",
	        8  => "Premium (Legacy)",
	        9  => "Professional (Legacy)",
	        11 => "Advanced Lite (Legacy)",
	        100 => "Starter V3 (Legacy)",
	        101 => "Growth V3 (Legacy)",
	        102 => "Advanced V3 (Legacy)",
	        103 => "Developer (Legacy)",
	        104 => "Startup (Legacy)",
	        105 => "Basic",
	        106 => "Plus",
	        107 => "Pro",
	        108 => "Elite"
	    ];
	    
	    return isset($licenseTypes[$permissionsProfileId]) ? $licenseTypes[$permissionsProfileId] : "Unknown";
	}
	
	public function setPermissions($str) 
	{
		$this->permissions = $str;
	}
	
	public function getPermissions() 
	{
		return $this->permissions;
	}

	public function setStatus($str)
	{
		$this->status = $str;
	}
	
	public function getStatus()
	{
		return $this->status;
	}
	
	public function setLastUpdated($str) 
	{
		$this->lastUpdated = $str;
	}
	
	public function getDateAdded() 
	{
		return $this->dateAdded;
	}
	
	public function setDateAdded($str) 
	{
		$this->dateAdded = $str;
	}
	
	public function getLastUpdated() 
	{
		return $this->lastUpdated;
	}
	
	public function setAsStaging($str)
	{
		$this->isStaging = $str;
	}
	
	public function isStaging() 
	{
		return ($this->isStaging == "1") ? true : false;
	}
	
	public function isArchived() 
	{
		return ($this->isArchivedFlag == "1") ? true : false;
	}
	
	public function setAsUnlicensed($isUnlicensed=true)
	{
	    $this->isUnlicensed = boolval($isUnlicensed);
	}
	
	public function isUnlicensed()
	{
	    return ($this->isUnlicensed === true);
	}
	
	
	public function isValid()
	{
	    //For compatibility
	    
	    //TODO: check status also?
	    return !$this->isUnlicensed();
	}
}
?>